﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.DragDrop;
using Telerik.Windows;
using System.Collections;
using Telerik.Windows.Controls.Animation;

namespace TileViewSwapOnDrop
{
	public partial class MainPage : UserControl
	{
		private bool isDragging;
		private MouseEventArgs mouseArgs;
		private Point startMousePosition;
		private RadTileViewItem draggedContainer;

		public MainPage()
		{
			InitializeComponent();
		}

		private void myTileView_PreviewTilePositionChanged(object sender, RadRoutedEventArgs e)
		{
			if (this.isDragging)
				e.Handled = true;
		}

		private void myTileView_TileDragStarted(object sender, TileViewDragEventArgs e)
		{
			this.startMousePosition = this.mouseArgs.GetPosition(null);
			this.draggedContainer = this.myTileView.ItemContainerGenerator.ContainerFromItem(e.DraggedItem) as RadTileViewItem;
			this.isDragging = true;
		}

		private void LayoutRoot_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
		{
			if (this.isDragging && this.draggedContainer != null)
			{
				this.isDragging = false;

				var endMousePosition = this.mouseArgs.GetPosition(null);
				var items = VisualTreeHelper.FindElementsInHostCoordinates(endMousePosition, this.myTileView);
				var itemToSwap = items.OfType<RadTileViewItem>().Where(i => i != this.draggedContainer).FirstOrDefault();
				if (itemToSwap != null)
				{
					var containerToSwap = this.myTileView.ItemContainerGenerator.ContainerFromItem(itemToSwap) as RadTileViewItem;
					this.EnsureDraggedContainer(endMousePosition);
					this.draggedContainer.Position = containerToSwap.Position;
					this.MoveItemsBetweenStartAndEnd(containerToSwap);
				}
			}
		}

		private void MoveItemsBetweenStartAndEnd(RadTileViewItem containerToSwap)
		{
			int offset = containerToSwap.Position - this.draggedContainer.Position;
			if (offset < 0)
			{
				for (int i = 0; i < Math.Abs(offset) - 1; i++)
				{
					containerToSwap.Position++;
				}
			}
			else
			{
				for (int i = 0; i < offset - 1; i++)
				{
					containerToSwap.Position--;
				}
			}
		}

		private void MoveItemsBetweenStartAndEnd(int draggPos, int swapPos)
		{
			int startPos = Math.Min(draggPos, swapPos);
			int endPos = Math.Max(draggPos, swapPos);
			List<RadTileViewItem> items;
			if (startPos == draggPos)
			{
				items = this.GetVisibleContainers(startPos, endPos - 2).OrderBy(i => i.Position).ToList();
				for (int i = 0; i < items.Count(); i++)
				{
					items[i].Position++;
				}
			}
			else
			{
				items = this.GetVisibleContainers(startPos, endPos - 1).OrderByDescending(i => i.Position).ToList();
				for (int i = 0; i < items.Count(); i++)
				{
					items[i].Position++;
				}
			}
		}

		private List<RadTileViewItem> GetVisibleContainers(int startPos, int endPos)
		{
			List<RadTileViewItem> items = new List<RadTileViewItem>();

			for (int i = 0; i <= this.myTileView.Items.Count; i++)
			{
				var container = this.myTileView.ItemContainerGenerator.ContainerFromIndex(i) as RadTileViewItem;
				if (container != null && container.Position >= startPos &&
					container.Position <= endPos)
				{
					items.Add(container);
				}
			}

			return items;
		}

		private void EnsureDraggedContainer(Point end)
		{
			if (this.draggedContainer != null)
			{
				var grid = this.draggedContainer.FindChildByType<Grid>();
				if (grid != null)
				{
					TransformGroup group = grid.RenderTransform as TransformGroup;
					if (group.Children.Count > 2)
					{
						(group.Children[3] as TranslateTransform).X -= this.startMousePosition.X - end.X;
						(group.Children[3] as TranslateTransform).Y -= this.startMousePosition.Y - end.Y;
					}
				}
			}
		}

		private void LayoutRoot_MouseMove(object sender, MouseEventArgs e)
		{
			this.mouseArgs = e;
		}

		private void Button_Click(object sender, RoutedEventArgs e)
		{
			var container = this.myTileView.ItemContainerGenerator.ContainerFromIndex(2) as RadTileViewItem;
			container.Position--;
		}
	}
}
